"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.generateDefaultFieldState = exports.createReducer = void 0;

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _immutable = require("immutable");

var _react = require("react");

var _fieldReducer = require("./fieldReducer");

var _useTextField = require("./fields/useTextField");

var _useSelectField = require("./fields/useSelectField");

var _useNumberField = require("./fields/useNumberField");

var _useBooleanField = require("./fields/useBooleanField");

var _formatLabel = require("./formatLabel");

var createReducer = function createReducer(_ref) {
  var fields = _ref.fields,
      _ref$options = _ref.options,
      options = _ref$options === void 0 ? {} : _ref$options,
      _ref$initialValues = _ref.initialValues,
      initialValues = _ref$initialValues === void 0 ? (0, _immutable.Map)() : _ref$initialValues;
  var derivedState = getInitialState(fields, initialValues, options);

  var _useReducer = (0, _react.useReducer)(_fieldReducer.fieldReducer, derivedState, _fieldReducer.initState),
      _useReducer2 = (0, _slicedToArray2.default)(_useReducer, 2),
      state = _useReducer2[0],
      dispatch = _useReducer2[1];

  var json = JSON.stringify(initialValues.toJS());
  (0, _react.useMemo)(function () {
    setTimeout(function () {
      dispatch(_fieldReducer.actions.reset(derivedState));
    });
    return json;
  }, [json]);
  return [state, dispatch];
};
/**
 * @param fields {Array<import("../form").IFormFieldArgs>}
 */


exports.createReducer = createReducer;

var getInitialState = function getInitialState(fields, initialValues, options) {
  var fieldMap = fields.reduce(function (acc, field) {
    return acc.set(field.name, generateDefaultFieldState(field, initialValues, options));
  }, (0, _immutable.Map)());
  return fieldMap;
};
/**
 *
 * @param field {import("../form").IFormFieldArgs}
 * @param {*} initialValues
 * @param options {import("../form").IFormOptions}
 */


var generateDefaultFieldState = function generateDefaultFieldState(field, initialValues, options) {
  if (!field.name) throw new Error('name is required on field');
  var _options$optionalLabe = options.optionalLabelFormatter,
      optionalLabelFormatter = _options$optionalLabe === void 0 ? _formatLabel.formatOptionalLabel : _options$optionalLabe;
  var label = field.optional ? optionalLabelFormatter(field.label, field.name) : field.label;
  var value = getInitialValue(field.value, initialValues.get(field.name), field.type);
  return (0, _immutable.fromJS)({
    initial: {
      type: field.type || 'text',
      value: value,
      optional: field.optional || false,
      label: label
    },
    current: {
      helperText: field.helperText || '',
      error: false,
      pristine: true,
      touched: false,
      value: value
    }
  }).setIn(['initial', 'field'], field);
};

exports.generateDefaultFieldState = generateDefaultFieldState;
var defaultValues = {
  text: _useTextField.defaultTextValue,
  select: _useSelectField.defaultSelectValue,
  number: _useNumberField.defaultNumberValue,
  boolean: _useBooleanField.defaultBooleanValue
};

var getInitialValue = function getInitialValue(fieldValue, initialValue) {
  var type = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : 'text';

  if (initialValue) {
    return initialValue;
  }

  if (fieldValue) {
    return fieldValue;
  }

  return defaultValues[type];
};