"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fieldReducer = exports.initState = exports.actions = exports.actionTypes = void 0;

var _defineProperty2 = _interopRequireDefault(require("@babel/runtime/helpers/defineProperty"));

var actionTypes = {
  insertField: 'insertField',
  removeField: 'removeField',
  reset: 'reset',
  touched: 'touched',
  updateValue: 'updateValue',
  validationResult: 'validationResult'
};
exports.actionTypes = actionTypes;
var actions = {
  insertField: function insertField(fieldName, fieldState) {
    return {
      type: actionTypes.insertField,
      fieldName: fieldName,
      payload: fieldState
    };
  },
  removeField: function removeField(fieldName) {
    return {
      type: actionTypes.removeField,
      fieldName: fieldName
    };
  },
  reset: function reset(state) {
    return {
      type: actionTypes.reset,
      payload: state
    };
  },
  touched: function touched(fieldName) {
    return {
      type: actionTypes.touched,
      fieldName: fieldName
    };
  },
  updateValue: function updateValue(fieldName, value) {
    return {
      type: actionTypes.updateValue,
      fieldName: fieldName,
      payload: value
    };
  },
  validationResult: function validationResult(fieldName, error, helperText) {
    return {
      type: actionTypes.validationResult,
      fieldName: fieldName,
      payload: {
        error: error,
        helperText: helperText
      }
    };
  }
};
exports.actions = actions;

var initState = function initState(state) {
  console.log('initing');
  return state;
};

exports.initState = initState;
var current = 'current';

var fieldReducer = function fieldReducer(state, _ref) {
  var _handlers;

  var type = _ref.type,
      fieldName = _ref.fieldName,
      payload = _ref.payload;
  var handlers = (_handlers = {}, (0, _defineProperty2.default)(_handlers, actionTypes.updateValue, function (value) {
    return state.setIn([fieldName, current, 'value'], value).setIn([fieldName, current, 'pristine'], value == state.getIn([fieldName, 'initial', 'value']));
  }), (0, _defineProperty2.default)(_handlers, actionTypes.touched, function () {
    return state.setIn([fieldName, current, 'touched'], true);
  }), (0, _defineProperty2.default)(_handlers, actionTypes.validationResult, function (_ref2) {
    var error = _ref2.error,
        helperText = _ref2.helperText;
    return state.setIn([fieldName, current, 'error'], error).setIn([fieldName, current, 'helperText'], helperText);
  }), (0, _defineProperty2.default)(_handlers, actionTypes.insertField, function (fieldState) {
    return state.set(fieldName, fieldState);
  }), (0, _defineProperty2.default)(_handlers, actionTypes.removeField, function () {
    return state.delete(fieldName);
  }), (0, _defineProperty2.default)(_handlers, actionTypes.reset, function (state) {
    return state;
  }), _handlers);
  var handler = handlers[type];

  if (handler) {
    return handler(payload);
  }

  return state;
};

exports.fieldReducer = fieldReducer;