"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useForm = void 0;

var _objectSpread2 = _interopRequireDefault(require("@babel/runtime/helpers/objectSpread"));

var _slicedToArray2 = _interopRequireDefault(require("@babel/runtime/helpers/slicedToArray"));

var _immutable = require("immutable");

var _Form = require("./Form");

var _createReducer3 = require("./createReducer");

var _fieldReducer = require("./fieldReducer");

var _fields = require("./fields");

/**
 * @param  param0 { import("../form").IFormProps }
 */
var useForm = function useForm(_ref) {
  var fields = _ref.fields,
      submit = _ref.submit,
      validate = _ref.validate,
      _ref$options = _ref.options,
      options = _ref$options === void 0 ? {} : _ref$options,
      _ref$initialValues = _ref.initialValues,
      initialValues = _ref$initialValues === void 0 ? (0, _immutable.Map)() : _ref$initialValues;

  var _createReducer = (0, _createReducer3.createReducer)({
    fields: fields,
    initialValues: initialValues,
    options: options
  }),
      _createReducer2 = (0, _slicedToArray2.default)(_createReducer, 2),
      state = _createReducer2[0],
      dispatch = _createReducer2[1];

  var fieldData = state.keySeq().reduce(function (acc, fieldName) {
    var fieldState = state.get(fieldName).set('getAllValues', function () {
      return getFieldValues(fieldData);
    });
    var fieldType = state.getIn([fieldName, 'initial', 'type']);
    var field = state.getIn([fieldName, 'initial', 'field']);

    switch (fieldType) {
      case 'select':
        acc[fieldName] = (0, _fields.useSelectField)(fieldState, dispatch, field);
        break;

      case 'boolean':
        acc[fieldName] = (0, _fields.useBooleanField)(fieldState, dispatch, field);
        break;

      case 'number':
        acc[fieldName] = (0, _fields.useNumberField)(fieldState, dispatch, field);
        break;

      case 'text':
      default:
        acc[fieldName] = (0, _fields.useTextField)(fieldState, dispatch, field);
        break;
    }

    return acc;
  }, {});

  var addField = function addField(field) {
    var fieldState = (0, _createReducer3.generateDefaultFieldState)(field, initialValues, options);
    dispatch(_fieldReducer.actions.insertField(field.name, fieldState));
  };

  var removeField = function removeField(fieldName) {
    dispatch(_fieldReducer.actions.removeField(fieldName));
  };

  var tryValidateForm = function tryValidateForm() {
    if (validate) {
      var values = getFieldValues(fieldData);
      return validate(values);
    }

    return {};
  };

  var trySubmitForm = function trySubmitForm() {
    var isFormValid = true;
    var formResults = tryValidateForm();
    var values = Object.entries(fieldData).reduce(function (acc, _ref2) {
      var _ref3 = (0, _slicedToArray2.default)(_ref2, 2),
          key = _ref3[0],
          v = _ref3[1];

      if (formResults[key]) {
        v.setValidationResult({
          error: true,
          helperText: formResults[key]
        });
        isFormValid = false;
      }

      var isFieldValid = v.validate();
      isFormValid = isFormValid && isFieldValid;
      return acc.set(key, v.props.value);
    }, initialValues);

    if (isFormValid) {
      submit(values);
    }
  };

  var setValue = function setValue(fieldName, value) {
    fieldData[fieldName].setValue(value);
  };

  return (0, _objectSpread2.default)({}, getFieldProps(fieldData), {
    setValue: setValue,
    submit: trySubmitForm,
    Form: _Form.Form,
    addField: addField,
    removeField: removeField
  });
};

exports.useForm = useForm;

var getFieldValues = function getFieldValues(fieldData) {
  return Object.entries(fieldData).reduce(function (acc, _ref4) {
    var _ref5 = (0, _slicedToArray2.default)(_ref4, 2),
        k = _ref5[0],
        v = _ref5[1];

    acc[k] = v.props.value;
    return acc;
  }, {});
};

var getFieldProps = function getFieldProps(fieldData) {
  return Object.entries(fieldData).reduce(function (acc, _ref6) {
    var _ref7 = (0, _slicedToArray2.default)(_ref6, 2),
        k = _ref7[0],
        v = _ref7[1];

    acc[k] = v.props;
    return acc;
  }, {});
};