"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.useFormField = void 0;

var _fieldReducer = require("./fieldReducer");

/**
 * @param  fieldArgs { import("../form").IFormFieldArgs}
 * @returns {import("../form").IFormField}
 */
var useFormField = function useFormField(state, dispatch) {
  var fieldArgs = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var requiredMessage = fieldArgs.requiredMessage || 'Required';

  var tryValidate = function tryValidate(value, touched) {
    var result;

    if (value && fieldArgs.validate) {
      result = fieldArgs.validate(value, fieldArgs.name, state.get('getAllValues'));
    }

    if (!Boolean(result) && !fieldArgs.optional && touched && value === '') {
      result = requiredMessage;
    }

    if (touched) {
      dispatch(_fieldReducer.actions.validationResult(fieldArgs.name, Boolean(result), result));
    }

    return !Boolean(result);
  };

  var setValue = function setValue(v) {
    dispatch(_fieldReducer.actions.updateValue(fieldArgs.name, v));
    tryValidate(v, state.getIn(['current', 'touched']));
  };

  var onChange = function onChange(_ref) {
    var target = _ref.target;
    var value = fieldArgs.valueFromTarget ? fieldArgs.valueFromTarget(target) : target.value;
    var coercedValue = fieldArgs.normalize ? fieldArgs.normalize(value) : value;
    setValue(coercedValue);
  };

  var onBlur = function onBlur() {
    dispatch(_fieldReducer.actions.touched(fieldArgs.name));
    tryValidate(state.getIn(['current', 'value']), true);
  };

  var setValidationResult = function setValidationResult(result) {
    dispatch(_fieldReducer.actions.validationResult(fieldArgs.name, true, result));
  };

  var validate = function validate() {
    return tryValidate(state.getIn(['current', 'value']), true);
  };

  return {
    props: {
      error: state.getIn(['current', 'error']),
      helperText: state.getIn(['current', 'helperText']),
      label: state.getIn(['initial', 'label']),
      value: state.getIn(['current', 'value']),
      // handlers
      onBlur: onBlur,
      onChange: onChange
    },
    setValidationResult: setValidationResult,
    setValue: setValue,
    validate: validate
  };
};

exports.useFormField = useFormField;